@@ -15,7 +15,7 @@ from socket import error as socket_error, EBADF, AF_INET, SOL_SOCKET, SO_REUSEAD
 
 from gevent.socket import socket, create_connection
 from gevent.coros import Semaphore
-from gevent import GreenletExit, spawn, Timeout, sleep
+from gevent import GreenletExit, spawn, Timeout, sleep, getcurrent
 from gevent.event import AsyncResult
 
 #from pickle import loads as pickle_loads, dumps as pickle_dumps, HIGHEST_PROTOCOL
@@ -96,6 +96,7 @@ ZIP_LENGTH = 1024 * 2 #if argkw > Nk, use zlib to compress
 ZIP_LEVEL = 3
 MAX_INDEX = (1 << 32) - 1
 
+
 use_logging = False
 log_head = '[p%s]' % os.getpid()
 def printf(msg, *args):
@@ -349,7 +350,8 @@ class RpcService(object):
         self._slock = Semaphore()
         self._reconnected = None
         self.reconnect_timeout = RECONNECT_TIMEOUT
-        self.iter_id = itertools.cycle(xrange(MAX_INDEX))
+        #self.iter_id = itertools.cycle(xrange(MAX_INDEX))
+        self._next_id = 0
         self._resps = {}
         self._proxys = WeakValueDictionary()
         self.stoped = True
@@ -359,6 +361,12 @@ class RpcService(object):
             self._heart_task = spawn(self.heartbeat)
         self.shells = {}
 
+    def next_id(self):
+        self._next_id += 1
+        if self._next_id >= MAX_INDEX:
+            self._next_id = 1
+        return self._next_id
+
     def start(self):
         if not self.stoped:
             return
@@ -581,7 +589,7 @@ class RpcService(object):
             argkw = zlib.compress(argkw, ZIP_LEVEL)
         if no_result:
             dtype |= ST_NO_RESULT
-        index = self.iter_id.next()
+        index = self.next_id() #iter_id.next()
         self.send(dtype, obj_id, index, name, argkw)
         if no_result:
             return
@@ -621,6 +629,8 @@ class RpcService(object):
                     proxys = _get_proxy(export_ids)
                 args = (proxys, ) + tuple(args[1:])
 
+            let = getcurrent()
+            setattr(let, _service_name_, self)
             if args is None:
                 rs = func()
             else:
@@ -1078,10 +1088,13 @@ def map_items_ex(proxys, attr, *args, **kw):
 
 
 
+_service_name_ = '_grpc_svc_'
 _my_servers_ = WeakValueDictionary()
 _addr_clients = WeakValueDictionary()
 _rpcs_ = []
 _services_ = []
+
+
 def uninit():
     """ clear all """
     global _my_servers_, _addr_clients
@@ -1095,6 +1108,10 @@ def uninit():
         svr.stop()
     #printf('_rpcs_:%s \n_services_:%s', _rpcs_, _services_)
 
+def get_service():
+    let = getcurrent()
+    return getattr(let, _service_name_, None)
+
 def get_rpc_by_addr(addr, force_rpc=0):
     global _addr_clients, _my_servers_
     if isinstance(addr, list):
